function [ExpectedValue,Volatility, Composition] = EfficientFrontier_ME_withVol(NumPortf, Covariance, ExpectedValues)

% This function returns the NumPortf x 1 vector expected returns,
%                       the NumPortf x 1 vector of volatilities and
%                       the NumPortf x NumAssets matrix of compositions
% of NumPortf efficient portfolios whose expected returns are equally spaced along the whole range of the efficient frontier

NumAssets=size(Covariance,2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine return of minimum-risk portfolio
FirstDegree=zeros(NumAssets,1);
SecondDegree=Covariance;
Aeq=ones(1,NumAssets);
beq=1;
A=-eye(NumAssets);          % no-short constraint
b=zeros(NumAssets,1);       % no-short constraint
x0=1/NumAssets*ones(NumAssets,1);
    options = optimoptions('quadprog',...
        'Algorithm','interior-point-convex','Display','off');
MinVol_Weights = quadprog(SecondDegree,FirstDegree,A,b,Aeq,beq,[],[],x0,options);
MinVol_Return=MinVol_Weights'*ExpectedValues;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine return of maximum-return portfolio
[MaxRet_Return,MaxRet_Index]=max(ExpectedValues);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% slice efficient frontier in NumPortf equally thick horizontal sectors in the upper branch only
Step=(MaxRet_Return-MinVol_Return)/(NumPortf-1);
TargetReturns=[MinVol_Return : Step : MaxRet_Return];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute the NumPortf compositions and risk-return coordinates of the optimal allocations relative to each slice

% start with min vol portfolio
Composition=MinVol_Weights';
Volatility=sqrt(MinVol_Weights'*Covariance*MinVol_Weights);
ExpectedValue=MinVol_Weights'*ExpectedValues;

for i=2:NumPortf-1
    
    % determine least risky portfolio for given expected return
    AEq=[ones(1,NumAssets);
        ExpectedValues'];
    bEq=[1
        TargetReturns(i)];
    options = optimoptions('quadprog',...
        'Algorithm','interior-point-convex','Display','off');
    Weights = quadprog(SecondDegree,FirstDegree,A,b,AEq,bEq,[],[],x0,options)';
    Composition=[Composition
        Weights];
    Volatility=[Volatility
        sqrt(Weights*Covariance*Weights')];
    ExpectedValue=[ExpectedValue
        Weights*ExpectedValues];
end
% add max ret portfolio
Weights=zeros(1,NumAssets);
Weights(MaxRet_Index)=1;
Composition=[Composition
    Weights];
Volatility=[Volatility
    sqrt(Weights*Covariance*Weights')];
ExpectedValue=[ExpectedValue
    Weights*ExpectedValues];


%% re-run using ME
Composition2 = [];
Volatility2 = [];
ExpectedValue2 = [];
fun = @(x) sum(x.*log(x));

for i=1:NumPortf
    
    % determine least risky portfolio for given expected return
    AEq=ones(1,NumAssets);
    bEq=1;
    
    A=-ExpectedValues';           % port return >= target return
    b=-ExpectedValue(i);
    
    mycon=@(wgt) deal(-wgt'*Covariance*wgt + Volatility(i)^2,[]);  % port var <= target var
    
    options = optimoptions('fmincon','Display','none','Algorithm','sqp');
    lb = zeros(NumAssets,1);
    ub = ones(NumAssets,1);
    x0=1/NumAssets*ones(NumAssets,1);
    
    Weights = fmincon(fun,x0,A,b,AEq,bEq,lb,ub,mycon,options)';
    Composition2=[Composition2
        Weights];
    Volatility2=[Volatility2
        sqrt(Weights*Covariance*Weights')];
    ExpectedValue2=[ExpectedValue2
        Weights*ExpectedValues];
end

end
